/*
   This file is part of the RELXILL model code.

   RELXILL is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   any later version.

   RELXILL is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.
   For a copy of the GNU General Public License see
   <http://www.gnu.org/licenses/>.

    Copyright 2018 Thomas Dauser, Remeis Observatory & ECAP

    Modified by Michal Szanecki (2018):
    removed parts related with relativistic transfer of radiation
*/
#include "xilbase.h"

const double ener_xill_norm_lo = 0.1;
const double ener_xill_norm_hi = 1000;
const int n_ener_xill = 3000;
double* ener_xill = NULL;

/** function adding a primary component with the proper norm to the flux **/
void add_primary_component(double* ener, int n_ener, double* flu, xillParam* xill_param, int* status){
  
	double pl_flux[n_ener];

	int ii;
	double pl_flux_xill[n_ener_xill];

	/** need to create an energy grid for the primary component to fulfill the XILLVER NORM condition (Dauser+2016) **/
	if (ener_xill == NULL){
		ener_xill = (double*) malloc( (n_ener_xill+1) * sizeof(double));
		CHECK_MALLOC_VOID_STATUS(ener_xill,status);
		get_log_grid(ener_xill,n_ener_xill+1,ener_xill_norm_lo,ener_xill_norm_hi);
	}

	/** note that in case of the nthcomp model Ecut is in the frame of the primary source
	    but for the bkn_powerlaw it is given in the observer frame */

	/** 1 **  calculate the primary spectrum  **/
	if (xill_param->prim_type == PRIM_SPEC_ECUT ){

		/** IMPORTANT: defintion of Ecut is ALWAYS in the frame of the observer by definition **/
		/**    (in case of the nthcomp primary continuum ect is actually kte ) **/
		double ecut_rest = xill_param->ect;

		for (ii=0; ii<n_ener_xill; ii++){
			pl_flux_xill[ii] = exp(1.0/ecut_rest) *
		             pow(0.5*(ener_xill[ii]+ener_xill[ii+1]),-xill_param->gam) *
		             exp( -0.5*(ener_xill[ii]+ener_xill[ii+1])/ecut_rest) *
		             (ener_xill[ii+1] - ener_xill[ii]);
		}

	} else if (xill_param->prim_type == PRIM_SPEC_NTHCOMP) {
		double nthcomp_param[5];
		/** important, kTe is given in the primary source frame, so we have to add the redshift here
		 *     however, only if the REL model **/
		double z=0.0;

		get_nthcomp_param(nthcomp_param, xill_param->gam, xill_param->ect, z);
		c_donthcomp(ener_xill, n_ener_xill, nthcomp_param, pl_flux_xill);
	} else {
		RELXILL_ERROR("trying to add a primary continuum to a model where this does not make sense (should not happen!)",status);
		return;
	}



	/** 2 **  get the normalization of the spectrum with respect to xillver **/
	/** everything is normalized to 10^15 cm^3 **/
	double norm_xill = 1e15 / 4.0 / M_PI;
	double keV2erg = 1.602177e-09;

	double sum_pl = 0.0;
	for (ii=0; ii<n_ener_xill; ii++){
	     sum_pl += pl_flux_xill[ii] * 0.5*(ener_xill[ii] + ener_xill[ii+1]) * 1e20 * keV2erg;
	}
	double norm_pl = norm_xill / sum_pl;

	/** bin the primary continuum onto the given grid **/
	rebin_spectrum( ener, pl_flux,n_ener, ener_xill, pl_flux_xill, n_ener_xill);

	for (ii=0; ii<n_ener; ii++){
	  pl_flux[ii] *= norm_pl;
	  flu[ii] *= fabs(xill_param->refl_frac);
	}
	/** 3 ** add power law component only if desired (i.e., refl_frac > 0)**/
	if (xill_param->refl_frac >= 0) {
	  for (ii=0; ii<n_ener; ii++) {
	    flu[ii] += pl_flux[ii];
	  }
	}

}



