/*
   This file is part of the RELXILL model code.

   RELXILL is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   any later version.

   RELXILL is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.
   For a copy of the GNU General Public License see
   <http://www.gnu.org/licenses/>.

    Copyright 2018 Thomas Dauser, Remeis Observatory & ECAP

    removed parts related with relativistic transfer of radiation 
    Modified by Michal Szanecki (2018) 
*/

#include "xilmodels.h"

int version_number_printed=0;

void print_version_number(int* status){

	if (version_number_printed==0){
		char *buf;
		get_version_number(&buf,status);
		printf("  *** loading XILLVER models from RELXILL ver. %s *** \n",buf);
		printf("  *** Garcia & Kallman (2010), ApJ, 718, 695 *** \n");
		printf("  *** Garcia et al. (2013), ApJ, 768, 2 *** \n");
		  
		free(buf);
		version_number_printed=1;
	}
}

xillParam* init_par_xillver(const double* inp_par, const int n_parameter, int* status){

	xillParam* param = new_xillParam(MOD_TYPE_XILLVER,PRIM_SPEC_ECUT,status);
	CHECK_STATUS_RET(*status,NULL);

	assert(n_parameter == NUM_PARAM_XILLVER);

	param->gam   = inp_par[0];
	param->afe   = inp_par[1];
	param->ect   = inp_par[2];
	param->lxi   = inp_par[3];
	param->dens  = 15;
	param->z     = inp_par[4];
	param->incl  = inp_par[5];
	param->refl_frac = inp_par[6];

	return param;
}

xillParam* init_par_xillver_dens(const double* inp_par, const int n_parameter, int* status){

	xillParam* param = new_xillParam(MOD_TYPE_XILLVERDENS,PRIM_SPEC_ECUT,status);
	CHECK_STATUS_RET(*status,NULL);

	assert(n_parameter == NUM_PARAM_XILLVER);

	param->gam   = inp_par[0];
	param->afe   = inp_par[1];
	param->ect   = 300.0;
	param->dens  = inp_par[2];
	param->lxi   = inp_par[3];
	param->z     = inp_par[4];
	param->incl  = inp_par[5];
	param->refl_frac = inp_par[6];

	return param;
}

xillParam* init_par_xillver_nthcomp(const double* inp_par, const int n_parameter, int* status){

	xillParam* param = new_xillParam(MOD_TYPE_XILLVER,PRIM_SPEC_NTHCOMP,status);
	CHECK_STATUS_RET(*status,NULL);

	assert(n_parameter == NUM_PARAM_XILLVER);

	param->gam   = inp_par[0];
	param->afe   = inp_par[1];
	param->ect   = inp_par[2];
	param->lxi   = inp_par[3];
	param->dens  = 15;
	param->z     = inp_par[4];
	param->incl  = inp_par[5];
	param->refl_frac = inp_par[6];

	return param;
}

/** shift the spectrum such that we can calculate the line for 1 keV **/
static double* shift_energ_spec_1keV(const double* ener, const int n_ener, double line_energ, double z,int* status){

	double* ener1keV = (double*) malloc((n_ener+1)*sizeof(double));
	CHECK_MALLOC_RET_STATUS(ener1keV,status,NULL);

	int ii;
	for (ii=0; ii<=n_ener; ii++){
		ener1keV[ii] = ener[ii]*(1 + z) / line_energ; //TODO: need to test this
	}
	return ener1keV;
}

/** XSPEC XILLVER MODEL FUNCTION **/
void tdxillver(const double* ener0, const int n_ener0, double* photar, const double* parameter, const int n_parameter, int* status){

	xillParam* param_struct = init_par_xillver(parameter,n_parameter,status);
	CHECK_STATUS_VOID(*status);

	xillver_base(ener0, n_ener0, photar, param_struct, status);
}

/** XSPEC XILLVER NTHCOMP MODEL FUNCTION **/
void tdxillver_nthcomp(const double* ener0, const int n_ener0, double* photar, const double* parameter, const int n_parameter, int* status){

	xillParam* param_struct = init_par_xillver_nthcomp(parameter,n_parameter,status);
	CHECK_STATUS_VOID(*status);

	xillver_base(ener0, n_ener0, photar, param_struct, status);
}


/** XSPEC XILLVER DENS MODEL FUNCTION **/
void tdxillverdens(const double* ener0, const int n_ener0, double* photar, const double* parameter, const int n_parameter, int* status){

	xillParam* param_struct = init_par_xillver_dens(parameter,n_parameter,status);
	CHECK_STATUS_VOID(*status);

	xillver_base(ener0, n_ener0, photar, param_struct, status);
}

/** BASIC XILLVER MODEL FUNCTION **/
void xillver_base(const double* ener0, const int n_ener0, double* photar, xillParam* param_struct, int* status){

	xill_spec* spec = get_xillver_spectra(param_struct,status);
	CHECK_STATUS_VOID(*status);

	assert(spec->n_incl==1);

	/** add the dependence on incl, assuming a semi-infinite slab **/
	norm_xillver_spec(spec, param_struct->incl);

	double * ener = (double*) ener0;
	int n_ener = (int) n_ener0;
	double flux[n_ener];

	rebin_spectrum( ener, flux,n_ener,spec->ener,spec->flu[0],spec->n_ener);

	int ii;

	for (ii=0; ii<n_ener0; ii++){
		flux[ii] /= pow(10,param_struct->lxi) ;
		if (fabs(param_struct->dens - 15) > 1e-6 ){
			flux[ii] /= pow(10,param_struct->dens - 15);
		}
	}

	add_primary_component(ener,n_ener,flux,param_struct, status);

	double* ener_shifted = shift_energ_spec_1keV(ener, n_ener, 1.0, param_struct->z,status);

	rebin_spectrum(ener_shifted, photar, n_ener, ener, flux, n_ener);

	/** free **/
	free_xillParam(param_struct);
	free_xill_spec(spec);
	free(ener_shifted);
}

/* get a new relbase parameter structure and initialize it */
xillParam* new_xillParam(int model_type, int prim_type, int* status){

	xillParam* param = (xillParam*) malloc(sizeof(xillParam));
	if (param==NULL){
		RELXILL_ERROR("memory allocation failed",status);
		return NULL;
	}
	param->model_type = model_type;
	param->prim_type = prim_type;

	param->gam = PARAM_DEFAULT;
	param->afe = PARAM_DEFAULT;
	param->lxi = PARAM_DEFAULT;
	param->ect = PARAM_DEFAULT;
	param->incl = PARAM_DEFAULT;
	param->z = PARAM_DEFAULT;
	param->refl_frac = PARAM_DEFAULT;
	param->fixReflFrac = -1;
	param->dens = PARAM_DEFAULT;

	return param;
}


/* free relbase parameter */
void free_xillParam(xillParam* param){

	free(param);
}

/** XSPEC XILLVER MODEL FUNCTION **/
void lmodxillver(const double* ener0, const int n_ener0, const double* parameter, int ifl, double* photar, double* photer, const char* init){

	const int n_parameter = 7;
	int status = EXIT_SUCCESS;
	tdxillver(ener0, n_ener0, photar, parameter, n_parameter, &status);

	if (status!=EXIT_SUCCESS)
	RELXILL_ERROR("evaluating xillver model failed",&status);
}

/** XSPEC XILLVER MODEL FUNCTION **/
void lmodxillverdens(const double* ener0, const int n_ener0, const double* parameter, int ifl, double* photar, double* photer, const char* init){

	const int n_parameter = 7;
	int status = EXIT_SUCCESS;
	tdxillverdens(ener0, n_ener0, photar, parameter, n_parameter, &status);

	if (status!=EXIT_SUCCESS)
	RELXILL_ERROR("evaluating xillver model failed",&status);
}

/** XSPEC XILLVER MODEL FUNCTION **/
void lmodxillvernthcomp(const double* ener0, const int n_ener0, const double* parameter, int ifl, double* photar, double* photer, const char* init){

	const int n_parameter = 7;
	int status = EXIT_SUCCESS;
	tdxillver_nthcomp(ener0, n_ener0, photar, parameter, n_parameter, &status);

	if (status!=EXIT_SUCCESS)
	RELXILL_ERROR("evaluating xillver model failed",&status);
}
